library IEEE;
use IEEE.STD_LOGIC_1164.all;
use IEEE.STD_LOGIC_ARITH.all;
use IEEE.STD_LOGIC_UNSIGNED.all;

entity hello is
  port (
    FPGA_CLK1 : in std_logic;           -- external clock
    
    PB_A0, PB_A1, PB_A2, PB_A3, PB_A4, PB_A5,
      PB_A6, PB_A7, PB_LB_N, PB_UB_N : out std_logic; --  bar leds
    
    PB_D0, PB_D1, PB_D2, PB_D3, PB_D4, PB_D5, PB_D6, PB_D7 : out std_logic;
    
    PB_D8, PB_D9, PB_D10, PB_D11,
    PB_D12, PB_D13, PB_D14, PB_D15 : out std_logic
    );  
end hello;

architecture imp of hello is

  component hex2led
    port (
      hex : in std_logic_vector(3 downto 0);
      led : out std_logic_vector(6 downto 0)
      );
  end component;
  
  signal cnt20 : std_logic_vector(19 downto 0);  -- clk divider
  signal cnt10 : std_logic_vector(9 downto 0);   -- our counter
  signal clk   : std_logic;                      -- our clock

  signal leftleds  : std_logic_vector(6 downto 0);
  signal rightleds : std_logic_vector(6 downto 0);

begin

  -- High-frequency clock divider

  process (FPGA_CLK1)
  begin
    if FPGA_CLK1'event and FPGA_CLK1 = '1' then  -- rising clock edge
      cnt20 <= cnt20 + 1;
    end if;
  end process;

  clk <= cnt20(19);

  -- Low-frequency clock divider

  process (clk)
  begin
    if clk'event and clk = '1' then              -- rising clock edge
      cnt10 <= cnt10 + 1;
    end if;
  end process;

  -- Two copies of the hex2led module, one for each 7-segment display

  l_h2l : hex2led
    port map (
      hex => cnt10(9 downto 6),
      led => leftleds
    );

  r_h2l : hex2led
    port map (
      hex => cnt10(5 downto 2),
      led => rightleds
    );

  -- Connecting the decoder modules to the appropriate ports

  ( PB_UB_N, PB_LB_N, PB_A7, PB_A6,
    PB_A5, PB_A4, PB_A3, PB_A2,  PB_A1, PB_A0 ) <= cnt10;
  
  ( PB_D6, PB_D5, PB_D4, PB_D3, PB_D2, PB_D1, PB_D0 ) <= leftleds;
  
  ( PB_D14, PB_D13, PB_D12, PB_D11, PB_D10, PB_D9, PB_D8 ) <= rightleds;
  
  PB_D7 <= '0';
  PB_D15 <= '0';            -- decimal points

end imp;
